/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package inject;

import com.sun.jersey.api.core.HttpContext;
import com.sun.jersey.core.spi.component.ComponentContext;
import com.sun.jersey.core.spi.component.ComponentScope;
import com.sun.jersey.spi.inject.Injectable;
import com.sun.jersey.spi.inject.InjectableProvider;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.lang.reflect.Type;
import javax.ws.rs.core.Context;
import javax.ws.rs.ext.Provider;

/**
 *
 * @author paulsandoz
 */
public class Injectors {

    public static interface Adder {
        void add(int a);

        int result();
    }

    private static class AdderImpl implements Adder {
        int s;

        public void add(int a) {
            s += a;
        }

        public int result() {
            return s;
        }

        public static Adder getInstance(HttpContext hc) {
            Adder a = (Adder)hc.getProperties().get(Adder.class.getName());
            if (a == null) {
                a = new AdderImpl();
                hc.getProperties().put(Adder.class.getName(), a);
            }
            return a;
        }
    }

    @Provider
    public static class AdderPerRequestProvider implements InjectableProvider<Context, Type> {

        public ComponentScope getScope() {
            return ComponentScope.PerRequest;
        }

        @Context HttpContext hc;
        
        public Injectable<Adder> getInjectable(ComponentContext ic, Context a, Type c) {
            if (c != Adder.class)
                return null;

            return new Injectable<Adder>() {
                public Adder getValue() {
                    return AdderImpl.getInstance(hc);
                }
            };
        }
    }


    @Provider
    public static class AdderSingletonProvider implements InjectableProvider<Context, Type> {

        public ComponentScope getScope() {
            return ComponentScope.Singleton;
        }

        @Context HttpContext hc;

        public Injectable<Adder> getInjectable(ComponentContext ic, Context a, Type c) {
            if (c != Adder.class)
                return null;
            
            return new Injectable<Adder>() {
                public Adder getValue() {
                    return (Adder)Proxy.newProxyInstance(
                                Adder.class.getClassLoader(),
                                new Class[] { Adder.class },
                                new InvocationHandler() {
                                    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
                                        Adder a = AdderImpl.getInstance(hc);
                                        return method.invoke(a, args);
                                    }
                                });
                }
            };
        }
    }
}
